@@ -1,7 +1,11 @@
 use super::*;
-use crate::util::{Result, WASH_CMD_INFO, WASH_LOG_INFO};
+use crate::ctl::{StartActorCommand, UpdateActorCommand};
+use crate::util::{Output, Result, WASH_CMD_INFO, WASH_LOG_INFO};
+use crossbeam_channel::Sender;
+use hotwatch::{Event, Hotwatch};
 use log::{debug, error, info};
-use std::sync::{mpsc::Sender, Arc, Mutex};
+use std::fs::metadata;
+use std::sync::{Arc, Mutex};
 use structopt::StructOpt;
 use termion::event::Key;
 use termion::{raw::RawTerminal, screen::AlternateScreen};
@@ -121,6 +125,78 @@ impl EmbeddedHost {
             op_sender,
         }
     }
+
+    /// Issues appropriate `ctl` commands to start the provided actors and create a
+    /// `Hotwatch` object for each of them. This `Hotwatch` object is responsible
+    /// for detecting write events and issuing a `CtlCliCommand::Update` when
+    /// an update occurs
+    pub(crate) fn watch_actors(&self, actors: Vec<PathBuf>) -> Vec<Hotwatch> {
+        actors
+            .iter()
+            .filter_map(|actor| match actor.to_str() {
+                Some(path) if std::fs::metadata(actor).is_ok() => match self.watch_actor(path) {
+                    Ok(hw) => Some(hw),
+                    Err(e) => {
+                        error!(target: WASH_CMD_INFO, "Unable to watch actor: {}", e);
+                        None
+                    }
+                },
+                _ => {
+                    error!(
+                        target: WASH_CMD_INFO,
+                        "Unable to watch actor: file does not exist"
+                    );
+                    None
+                }
+            })
+            .collect::<Vec<_>>()
+    }
+
+    fn watch_actor(&self, actor_ref: &str) -> Result<Hotwatch> {
+        debug!(
+            target: WASH_CMD_INFO,
+            "Attempting to watch actor {}", actor_ref
+        );
+        let start_cmd = CtlCliCommand::Start(StartCommand::Actor(StartActorCommand::new(
+            ConnectionOpts::default(),
+            Output::default(),
+            Some(self.id.to_string()),
+            actor_ref.to_string(),
+            None,
+            1,
+        )));
+        self.op_sender.send(start_cmd)?;
+
+        let actor = Actor::from_file(&actor_ref).map_err(convert_error)?;
+
+        // Repeated updates with the same actor can re-use the same command
+        let update_cmd = CtlCliCommand::Update(UpdateCommand::Actor(UpdateActorCommand::new(
+            ConnectionOpts::default(),
+            Output::default(),
+            self.id.to_string(),
+            actor.public_key(),
+            actor_ref.to_string(),
+        )));
+
+        let op_sender = self.op_sender.clone();
+        let actor_ref = actor_ref.to_string();
+        let mut hotwatch = Hotwatch::new().expect("hotwatch failed to initialize!");
+        hotwatch
+            .watch(actor_ref.clone(), move |event: Event| {
+                // Watching for Event::NoticeWrite is faster, but it induces a race condition where
+                // an update _could_ take place _before_ the write has finished.
+                if let Event::Write(_path) = event {
+                    info!(
+                        target: WASH_CMD_INFO,
+                        "Detected actor change for actor {}, updating", actor_ref
+                    );
+                    let _ = op_sender.send(update_cmd.clone());
+                }
+            })
+            .expect("failed to watch file");
+
+        Ok(hotwatch)
+    }
 }
 
 pub(crate) struct WashRepl {
@@ -284,7 +360,7 @@ impl WashRepl {
                             ReplCliCommand::Ctl(ctlcmd) => {
                                 // This match statement handles loading an actor from disk instead of from an OCI registry
                                 //
-                                // When a StartActor `ctl` command is sent, we send the `ctl` command to the host API for the following cases:
+                                // When a Start or Update Actor `ctl` command is sent, we send the `ctl` command to the host API for the following cases:
                                 // 1. The Host is running in standalone mode (all ctl commands are delegated to host API)
                                 // 2. The actor_ref exists as a file on disk AND:
                                 //    a. The host ID specified is the embedded host
@@ -294,9 +370,20 @@ impl WashRepl {
                                         Some(host),
                                         CtlCliCommand::Start(StartCommand::Actor(cmd)),
                                     ) if host.mode == ReplMode::Lattice => {
-                                        if std::fs::metadata(&cmd.actor_ref).is_ok() // File exists
-                                                && (cmd.host_id.is_none()
-                                                    || cmd.host_id.unwrap() == host.id)
+                                        if metadata(&cmd.actor_ref).is_ok() // File exists
+                                            && (cmd.host_id.is_none()
+                                                || cmd.host_id.unwrap() == host.id)
+                                        {
+                                            host.op_sender.send(ctlcmd)?;
+                                            return Ok(());
+                                        }
+                                    }
+                                    (
+                                        Some(host),
+                                        CtlCliCommand::Update(UpdateCommand::Actor(cmd))
+                                    ) if host.mode == ReplMode::Lattice => {
+                                        if metadata(&cmd.new_actor_ref).is_ok() // File exists
+                                            && cmd.host_id == host.id
                                         {
                                             host.op_sender.send(ctlcmd)?;
                                             return Ok(());
